//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Sound_Library v1.4
//  Example1 - Enumerating Sound Devices
//
// Date: 17th May, 2001
// Author: Darren John Dwyer
// Copyright: (c) 2001 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to enumerate available DirectSound devices,
// then how to create a selected device, retrieve it's capabilites,
// test it's playback, and finally, how to destroy the device and cleanup.
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSCaps"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;


//---------------------------------------------------------------------------
// DEBUGGING
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }


//---------------------------------------------------------------------------
// MAINFORM
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Top=0;
Left=0;
Width=800;
Height=600;
Guids = new TList();
Devices = new TStringList();
Modules = new TStringList();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
// note: make sure to remove all TDx_Sound devices here
if (Dx_Sound1->Created) Dx_Sound1->Destroy();

if (Guids) delete Guids;
if (Modules) delete Modules;
if (Devices) delete Devices;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormPaint(TObject *Sender)
{
Canvas->Brush->Color = clBlack;
Canvas->Brush->Style = bsSolid;
Canvas->FrameRect( ClientRect );
}



//---------------------------------------------------------------------------
// BUTTONS
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumClick(TObject *Sender)
{
Msg( "Enumerating DirectSound devices..." );

Guids->Clear();
Devices->Clear();
Modules->Clear();
DevicesListBox->Items->Clear();

// Note: the call to DSEnumerate() immediately below this comment
// triggers the TMainForm::Dx_Sound1DSEnumerate() event (at bottom of code)
// a number of times in quick succession, then returns to the line of code
// immediately following the DSEnumerate() call.

if (!Dx_Sound1->DSEnumerate( NULL )) Msg("No available DirectSound devices!");

ActiveDeviceLabel->Caption = "none";
TotalDevicesLabel->Caption = IntToStr(Devices->Count);

Msg( "Found: "+IntToStr(Devices->Count)+" DirectSound Devices" );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NextClick(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    if (DevicesListBox->ItemIndex==DevicesListBox->Items->Count-1) DevicesListBox->ItemIndex=0;
    else DevicesListBox->ItemIndex = DevicesListBox->ItemIndex + 1;
    Msg( "Selecting Device: "+Devices->Strings[DevicesListBox->ItemIndex] );
    }
else
    DevicesListBox->ItemIndex = -1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TestClick(TObject *Sender)
{
DevicesListBox->OnClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}



//---------------------------------------------------------------------------
// CLICKS, ETC.
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (DevicesListBox->ItemIndex==-1) return;

GUID* guid = (GUID*) Guids->Items[DevicesListBox->ItemIndex];
AnsiString device_name = Devices->Strings[DevicesListBox->ItemIndex];

if (Dx_Sound1->Created) Dx_Sound1->Destroy();
if (Dx_Sound1->Create( guid ))
    {
    Msg( "Testing Device: "+device_name );
    ActiveDeviceLabel->Caption = device_name;
    }
else ActiveDeviceLabel->Caption = "none";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ButtonsMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
if (!Active) return;
TComponent* sender = (TComponent*) Sender;
if (sender==NULL) return;

// if there is an image that is already Highlighted, un-highlight it now.
if ( !sender->Name.Pos("Enum") && Highlighted.Pos("Enum")) { EnumImage->Picture = Lo->Picture; Enum->Font->Color = clNavy; }
else if ( !sender->Name.Pos("Next") && Highlighted.Pos("Next")) { NextImage->Picture = Lo->Picture; Next->Font->Color = clNavy; }
else if ( !sender->Name.Pos("Test") && Highlighted.Pos("Test")) { TestImage->Picture = Lo->Picture; Test->Font->Color = clNavy; }
else if ( !sender->Name.Pos("Exit") && Highlighted.Pos("Exit")) { ExitImage->Picture = Lo->Picture; Exit->Font->Color = clNavy; }
else if ( !sender->Name.Pos("Clear") && Highlighted.Pos("Clear")) { ClearImage->Picture = Lo->Picture; Clear->Font->Color = clNavy; }

// if the image is not already Highlighted, but now needs to be Highlighted....
if ( sender->Name.Pos("Enum") && !Highlighted.Pos("Enum")) { EnumImage->Picture = Hi->Picture; Enum->Font->Color = clWhite; }
else if ( sender->Name.Pos("Next") && !Highlighted.Pos("Next")) { NextImage->Picture = Hi->Picture; Next->Font->Color = clWhite; }
else if ( sender->Name.Pos("Test") && !Highlighted.Pos("Test")) { TestImage->Picture = Hi->Picture; Test->Font->Color = clWhite; }
else if ( sender->Name.Pos("Exit") && !Highlighted.Pos("Exit")) { ExitImage->Picture = Hi->Picture; Exit->Font->Color = clWhite; }
else if ( sender->Name.Pos("Clear") && !Highlighted.Pos("Clear")) { ClearImage->Picture = Hi->Picture; Clear->Font->Color = clWhite; }

// remember which button is Highlighted
Highlighted = sender->Name;
}



//---------------------------------------------------------------------------
// TDX_SOUND_LIBRARY
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
if (Dx_Sound1->GetCaps( DSCaps1 )) CapsListBox->Items->Text = DSCaps1->Flags_Strings->Text;
Dx_Sound1->SetCooperativeLevel( Application->Handle, DSSCL_PRIORITY );
Dx_SoundBuffer1->CreateFromFile( "TDx_Library.wav", DSBufferDesc1, Dx_Sound1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
CapsListBox->Items->Clear();
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Create(TObject *Sender)
{
Dx_SoundBuffer1->Play( 0,0 );
if (Dx_SoundBuffer1->Playing)
    {
    Msg( "Device Test: OK" );
    }
else
    {
    Msg( "Device Test: MAY HAVE FAILED." );
    Msg( "It could be that there is too much latency and the test sound has not yet begun playback." );
    Msg( "If you hear a clear sound, the device is working OK." );
    Msg( "If you did not hear a sound, there should be an error message below explaining why the device is not working correctly." );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
Dx_SoundBuffer1->CreateFromFile( "TDx_Library.wav", DSBufferDesc1, Dx_Sound1 );
Dx_SoundBuffer1->Play(0,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1DSEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverModule, void *Context,
    bool &Finished)
{
Msg( "Found Device: "+DriverDescription );
if (Devices && Modules && Guids)
	{
    Devices->Add( DriverDescription );
    Modules->Add( DriverModule );
    Guids->Add( (void*) Guid );

    DevicesListBox->Items->Add( DriverDescription+", ("+DriverModule+")" );
    }
else
	Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Error(TObject *Sender,
      AnsiString Function, AnsiString Error,
      AnsiString ErrorMessage)
{
Msg( "--------------" );
Msg( "Error in: "+Function );
Msg( "Error code: "+Error );
Msg( "Error message: "+ErrorMessage );
}
//---------------------------------------------------------------------------




