//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Sound_Library v1.4
//  ExampleA - Capturing Sounds
//
// Date: 17th May, 2001
// Author: Darren John Dwyer
// Copyright: (c) 2001 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to enumerate DirectSoundCapture recording devices,
// then create either the default device, or any particular recording device,
// then record sound using this device,
// and then playback the recording through an identical format sound output buffer,
// copying the data directly from the sound capture buffer to the sound buffer.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
//---------------------------------------------------------------------------
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 or 4 only
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundCapture"
#pragma link "TDx_SoundCaptureBuffer"
#pragma link "TDSCBufferDesc"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSBufferDesc"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
int INDENT = 0;
#define Msg(A) 	try {\
					AnsiString ssss = "";\
                    for (int nnnn=0;nnnn<INDENT*4;nnnn++) ssss = ssss + " ";\
                    ssss = ssss + (A);\
                    if (StatusMemo)\
                        if (StatusMemo->Lines)\
                            {\
                            if (StatusMemo->Lines->Text.Length()>10000) StatusMemo->Lines->Clear();\
							StatusMemo->Lines->Add( ssss );\
                            }\
                    }\
				catch (Exception &E)\
                	{\
                    if (StatusMemo)\
                        if (StatusMemo->Lines)\
                    		StatusMemo->Lines->Clear();\
                    }\
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
NumCaptureDevices = 0;
DevicesListBox->Items->Clear();
Dx_SoundCapture1->DSCaptureEnumerate(NULL);

Dx_Sound1->Create(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCapture1DSCaptureEnumerate(
      TObject *Sender, GUID *Guid, AnsiString DriverDescription,
      AnsiString DriverModule, void *Context, bool &Finished)
{
if (NumCaptureDevices<MAX_DEVICES_SUPPORTED)
    {
    Guids[NumCaptureDevices] = Guid;
    DriverDescriptions[NumCaptureDevices] = DriverDescription;
    DriverModules[NumCaptureDevices] = DriverModule;

    NumCaptureDevices++;
    DevicesListBox->Items->Add( DriverDescription+" ["+DriverModule+"]" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
if (Dx_SoundCapture1->Created) Dx_SoundCapture1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (Dx_SoundCapture1->Created) Dx_SoundCapture1->Destroy();

if (DevicesListBox->ItemIndex==-1)
    Dx_SoundCapture1->Create(NULL);
else
    Dx_SoundCapture1->Create(Guids[DevicesListBox->ItemIndex]);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCapture1Create(TObject *Sender)
{
// we want to use a new recording buffer each time, so destroy any old ones still existing
if (Dx_SoundCaptureBuffer1->Created) Dx_SoundCaptureBuffer1->Destroy();

// setup desired format for the DirectSoundCapture recording buffer.
int seconds = SecondsScrollBar->Position;
int channels = NumChannelsRadioGroup->ItemIndex + 1;
int frequency =  11025 << FrequencyRadioGroup->ItemIndex;  // 11025, 22050, 44100
int bits = (BitsRadioGroup->ItemIndex+1) * 8;

WaveFormat.cbSize = sizeof(WAVEFORMATEX) + 0;       // 0 extra bytes at end of WAVEFORMATEX for WAVE_FORMAT_PCM
WaveFormat.wFormatTag = WAVE_FORMAT_PCM;            // DirectSound only supports WAVE_FORMAT_PCM, an uncompressed format.
WaveFormat.nChannels = (WORD) channels;             // # of channels, mono==1, stereo==2
WaveFormat.nSamplesPerSec = (DWORD) frequency;      // recording frequency, 11025 Hz
WaveFormat.wBitsPerSample = (WORD) bits;            // 8 bit data, 16 bit data
int temp = (WaveFormat.nChannels * WaveFormat.wBitsPerSample) / 8;                  // required calculation ..
WaveFormat.nBlockAlign = (WORD) temp;                                               // .. to setup raw data block alignment ..
WaveFormat.nAvgBytesPerSec = WaveFormat.nSamplesPerSec * WaveFormat.nBlockAlign;    // .. and average samples per second.

// setup buffer of total # of seconds * bytes per second
// and setup the required format
DSCBufferDesc1->Clear();
DSCBufferDesc1->BufferBytes = seconds * WaveFormat.nAvgBytesPerSec;
DSCBufferDesc1->Format = &WaveFormat;

Dx_SoundCaptureBuffer1->Create( DSCBufferDesc1, Dx_SoundCapture1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCapture1Destroy(TObject *Sender)
{
//Msg("Sound Capture interface is being destroyed");
if (Dx_SoundCaptureBuffer1->Created) Dx_SoundCaptureBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RecordButtonClick(TObject *Sender)
{
if (Dx_SoundCaptureBuffer1->Start(0))
    {
    StatusLabel->Caption = "- Started Recording -";
    StatusLabel->Update();
    }

while (Dx_SoundCaptureBuffer1->Recording)
    {
	dword current_capture_pos;
	dword current_read_pos;
    Dx_SoundCaptureBuffer1->GetCurrentPosition( &current_capture_pos, &current_read_pos);

// handy values...

    dword buffer_size = DSCBufferDesc1->BufferBytes;
    dword average_bytes_per_second = DSCBufferDesc1->Format->nAvgBytesPerSec;

//    dword format_tag = DSCBufferDesc1->Format->wFormatTag;
//    dword number_of_channels = DSCBufferDesc1->Format->nChannels;
//    dword samples_per_second = DSCBufferDesc1->Format->nSamplesPerSec;
//    dword block_alignment = DSCBufferDesc1->Format->nBlockAlign;
//    dword bits_per_sample = DSCBufferDesc1->Format->wBitsPerSample;
//    dword bytes_remaining =  buffer_size - current_capture_pos;

    float time_total = (float) buffer_size / (float) average_bytes_per_second;
    float percentage = (float) current_capture_pos / (float) buffer_size;
    float time_elapsed = percentage * time_total;

//    float other_percentage = 1.0 - percentage;
//    float time_remaining =  other_percentage * time_total;

    SecondsLabel->Caption = "- "+FloatToStrF(time_elapsed,ffFixed,2,2)+" seconds -";
    SecondsLabel->Update();

    StatusLabel->Caption = "- Recording -";
    StatusLabel->Update();
    Application->ProcessMessages();
    }

StatusLabel->Caption = "- Finished Recording -";
StatusLabel->Update();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopButtonClick(TObject *Sender)
{
Dx_SoundCaptureBuffer1->Stop();
Msg("Recording Stopped.");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlaybackButtonClick(TObject *Sender)
{
// setup ready to create the DirectSoundBuffer based on the DirectSoundCaptureBuffer format details... cool huh?
DSBufferDesc1->Clear();
DSBufferDesc1->Flags = DSBCAPS_STATIC | DSBCAPS_GLOBALFOCUS;
DSBufferDesc1->Format = DSCBufferDesc1->Format;
DSBufferDesc1->BufferBytes = DSCBufferDesc1->BufferBytes;

// create the required format DirectSoundBuffer
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
if (Dx_SoundBuffer1->Create( DSBufferDesc1, Dx_Sound1 ))
    {
    // copy the raw sound data from the capture buffer to the output buffer
    void* srce;         // used for byte storage
    void* srce_dummy;   // not used cause we're locking the entire buffer in one go.
    void* dest;
    void* dest_dummy;
    dword srce_bytes;
    dword srce_bytes_dummy;
    dword dest_bytes;
    dword dest_bytes_dummy;

    if (Dx_SoundCaptureBuffer1->Lock( 0, DSCBufferDesc1->BufferBytes, &srce, &srce_bytes, &srce_dummy, &srce_bytes_dummy, DSCBLOCK_ENTIREBUFFER ))
        {
        if (Dx_SoundBuffer1->Lock( 0, DSBufferDesc1->BufferBytes, &dest, &dest_bytes, &dest_dummy, &dest_bytes_dummy, DSBLOCK_ENTIREBUFFER ))
            {
            Msg("Copying Data from Sound Capture Buffer to Sound Playback Buffer");
            CopyMemory( dest, srce, dest_bytes );

            Dx_SoundBuffer1->Unlock( dest, dest_bytes, dest_dummy, dest_bytes_dummy );
            }

        Dx_SoundCaptureBuffer1->Unlock( srce, srce_bytes, srce_dummy, srce_bytes_dummy );
        }

    // now play back the sound buffer
    if (Dx_SoundBuffer1->Play(0,0))
        {
        StatusLabel->Caption = "- Started Playback -";
        StatusLabel->Update();
        }

    while (Dx_SoundBuffer1->Playing)
        {
        dword current_play_pos;
        dword current_write_pos;
        Dx_SoundBuffer1->GetCurrentPosition( &current_play_pos, &current_write_pos);

    // handy values...

        dword buffer_size = DSBufferDesc1->BufferBytes;
        dword average_bytes_per_second = DSBufferDesc1->Format->nAvgBytesPerSec;

    //    dword format_tag = DSBufferDesc1->Format->wFormatTag;
    //    dword number_of_channels = DSBufferDesc1->Format->nChannels;
    //    dword samples_per_second = DSBufferDesc1->Format->nSamplesPerSec;
    //    dword block_alignment = DSBufferDesc1->Format->nBlockAlign;
    //    dword bits_per_sample = DSBufferDesc1->Format->wBitsPerSample;
    //    dword bytes_remaining =  buffer_size - current_play_pos;

        float time_total = (float) buffer_size / (float) average_bytes_per_second;
        float percentage = (float) current_play_pos / (float) buffer_size;
        float time_elapsed = percentage * time_total;

    //    float other_percentage = 1.0 - percentage;
    //    float time_remaining =  other_percentage * time_total;

        SecondsLabel->Caption = "- "+FloatToStrF(time_elapsed,ffFixed,2,2)+" seconds -";
        SecondsLabel->Update();

        StatusLabel->Caption = "- Playing -";
        StatusLabel->Update();
        Application->ProcessMessages();
        }

    StatusLabel->Caption = "- Finished Playback -";
    StatusLabel->Update();
    Msg("DONE...");
    Msg("");
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCapture1Error(TObject *Sender,
    AnsiString Function, AnsiString Error, AnsiString ErrorMessage)
{
Msg("----------------------------");
Msg("Dx_SoundCapture1->Error:");
Msg("----------------------------");
Msg("Error: "+Error);
Msg("Function: "+Function);
Msg("Message:  "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCaptureBuffer1Error(TObject *Sender,
    AnsiString Function, AnsiString Error, AnsiString ErrorMessage)
{
Msg("----------------------------");
Msg("Dx_SoundCaptureBuffer1->Error:");
Msg("----------------------------");
Msg("Error: "+Error);
Msg("Function: "+Function);
Msg("Message:  "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("----------------------------");
Msg("Dx_Sound1->Error:");
Msg("----------------------------");
Msg("Error: "+Error);
Msg("Function: "+Function);
Msg("Message:  "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Error(TObject *Sender,
    AnsiString Function, AnsiString Error, AnsiString ErrorMessage)
{
Msg("----------------------------");
Msg("Dx_SoundBuffer1->Error:");
Msg("----------------------------");
Msg("Error: "+Error);
Msg("Function: "+Function);
Msg("Message:  "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCaptureBuffer1Create(TObject *Sender)
{
Msg("Sound Capture Buffer has been created");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundCaptureBuffer1Destroy(TObject *Sender)
{
Msg("Sound Capture Buffer is being destroyed");
Dx_SoundCaptureBuffer1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
Msg("Sound Playback interface has been created");
Dx_Sound1->SetCooperativeLevel( Application->Handle, DSSCL_EXCLUSIVE );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
Msg("Sound Playback interface is being destroyed" );
if (Dx_SoundBuffer1->Created)
    Dx_SoundBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1BufferLost(TObject *Sender)
{
Msg("Sound Playback Buffer has been lost");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Create(TObject *Sender)
{
Msg( "Sound Playback Buffer has been created");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Destroy(TObject *Sender)
{
Msg( "Sound Playback Buffer is being destroyed");
if (Dx_SoundBuffer1->Playing) Dx_SoundBuffer1->Stop();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SecondsScrollBarChange(TObject *Sender)
{
AnsiString temp = "- "+IntToStr(SecondsScrollBar->Position)+" second";
if (SecondsScrollBar->Position>1) temp = temp + "s";
temp = temp + " -";
TotalSecondsLabel->Caption = temp;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button1Click(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    if (DevicesListBox->ItemIndex==DevicesListBox->Items->Count-1) DevicesListBox->ItemIndex=0;
    else DevicesListBox->ItemIndex = DevicesListBox->ItemIndex + 1;
    DevicesListBox->OnClick(this);
    }
else
    DevicesListBox->ItemIndex = -1;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::StopPlayButtonClick(TObject *Sender)
{
Dx_SoundBuffer1->Stop();
Msg("Playback Stopped.");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button2Click(TObject *Sender)
{
if (!Dx_SoundCaptureBuffer1->Created) return;

AnsiString filename = GetCurrentDir() + "\\ExampleA_Capture.wav";
try {
    if (FileExists( filename ))
        {
        Msg( "Deleting existing file: "+filename );
        DeleteFile( filename.c_str() );
        }
    Msg( "Saving Recording to file: "+filename );

    Dx_SoundCaptureBuffer1->StreamSize = DSCBufferDesc1->BufferBytes;
    Dx_SoundCaptureBuffer1->SaveToFile( filename, &WaveFormat, sizeof(WAVEFORMATEX) );
    }
catch (...)
    {
    Msg( "Error:: Could not save to file: "+filename );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Button3Click(TObject *Sender)
{
// create the required format DirectSoundBuffer
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
if (Dx_SoundBuffer1->CreateFromFile( GetCurrentDir() + "\\ExampleA_Capture.wav", DSBufferDesc1, Dx_Sound1 ))
    {
    // now play back the sound buffer
    if (Dx_SoundBuffer1->Play(0,0))
        {
        StatusLabel->Caption = "- Started Playback -";
        StatusLabel->Update();
        }

    while (Dx_SoundBuffer1->Playing)
        {
        dword current_play_pos;
        dword current_write_pos;
        Dx_SoundBuffer1->GetCurrentPosition( &current_play_pos, &current_write_pos);

    // handy values...

        dword buffer_size = DSBufferDesc1->BufferBytes;
        dword average_bytes_per_second = DSBufferDesc1->Format->nAvgBytesPerSec;

    //    dword format_tag = DSBufferDesc1->Format->wFormatTag;
    //    dword number_of_channels = DSBufferDesc1->Format->nChannels;
    //    dword samples_per_second = DSBufferDesc1->Format->nSamplesPerSec;
    //    dword block_alignment = DSBufferDesc1->Format->nBlockAlign;
    //    dword bits_per_sample = DSBufferDesc1->Format->wBitsPerSample;
    //    dword bytes_remaining =  buffer_size - current_play_pos;

        float time_total = (float) buffer_size / (float) average_bytes_per_second;
        float percentage = (float) current_play_pos / (float) buffer_size;
        float time_elapsed = percentage * time_total;

    //    float other_percentage = 1.0 - percentage;
    //    float time_remaining =  other_percentage * time_total;

        SecondsLabel->Caption = "- "+FloatToStrF(time_elapsed,ffFixed,2,2)+" seconds -";
        SecondsLabel->Update();

        StatusLabel->Caption = "- Playing -";
        StatusLabel->Update();
        Application->ProcessMessages();
        }

    StatusLabel->Caption = "- Finished Playback -";
    StatusLabel->Update();
    Msg("DONE...");
    Msg("");
    }
}
//---------------------------------------------------------------------------
